# Copyright (c) 2014 Rackspace Hosting.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from designateclient import exceptions as designateclient

from openstack_dashboard import exceptions

NOT_FOUND = exceptions.NOT_FOUND + (
    designateclient.ResourceNotFound,
    designateclient.NotFound,
    )
RECOVERABLE = exceptions.RECOVERABLE + (
    designateclient.BadRequest,
    designateclient.Conflict,
    )
UNAUTHORIZED = exceptions.UNAUTHORIZED + (
    designateclient.Forbidden,
    )



# Copyright 2012 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
# Copyright 2012 Nebula, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from designatedashboard.tests import base

DOMAIN_ID = '123'


class ARecordFormTests(base.BaseRecordFormCleanTests):

    IPV4 = '1.1.1.1'

    MSG_INVALID_IPV4 = 'Enter a valid IPv4 address'

    def setUp(self):
        super(ARecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'A'
        self.form.cleaned_data['name'] = self.HOSTNAME
        self.form.cleaned_data['data'] = self.IPV4

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_valid_name_field_wild_card(self):
        self.form.cleaned_data['name'] = '*'
        self.form.clean()
        self.assert_no_errors()

    def test_missing_name_field(self):
        self.form.cleaned_data['name'] = ''
        self.form.clean()
        self.assert_no_errors()
        self.assertIsNotNone(self.form.cleaned_data['name'])

    def test_missing_data_field(self):
        self.form.cleaned_data['data'] = ''
        self.form.clean()
        self.assert_required_error('data')

    def test_invalid_name_field(self):
        self.form.cleaned_data['name'] = '$#%foo!!'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_starting_dash(self):
        self.form.cleaned_data['name'] = '-ww'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_trailing_dash(self):
        self.form.cleaned_data['name'] = 'co-'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_bad_wild_card(self):
        self.form.cleaned_data['name'] = 'derp.*.' + self.DOMAIN_NAME
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_outside_of_domain_name_field(self):
        self.form.cleaned_data['name'] = 'www.bar.com.'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_data_field(self):
        self.form.cleaned_data['data'] = 'foo'
        self.form.clean()
        self.assert_error('data', self.MSG_INVALID_IPV4)


class AAAARecordFormTests(base.BaseRecordFormCleanTests):

    IPV6 = '1111:1111:1111:11::1'

    MSG_INVALID_IPV6 = 'Enter a valid IPv6 address'

    def setUp(self):
        super(AAAARecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'AAAA'
        self.form.cleaned_data['name'] = self.HOSTNAME
        self.form.cleaned_data['data'] = self.IPV6

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_valid_name_field_wild_card(self):
        self.form.cleaned_data['name'] = '*'
        self.form.clean()
        self.assert_no_errors()

    def test_missing_name_field(self):
        self.form.cleaned_data['name'] = ''
        self.form.clean()
        self.assert_no_errors()
        self.assertIsNotNone(self.form.cleaned_data['name'])

    def test_missing_data_field(self):
        self.form.cleaned_data['data'] = ''
        self.form.clean()
        self.assert_required_error('data')

    def test_invalid_name_field(self):
        self.form.cleaned_data['name'] = '#@$foo!!'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_starting_dash(self):
        self.form.cleaned_data['name'] = '-ww.foo.com'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_trailing_dash(self):
        self.form.cleaned_data['name'] = 'www.foo.co-'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_bad_wild_card(self):
        self.form.cleaned_data['name'] = 'derp.*.' + self.DOMAIN_NAME
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_outside_of_domain_name_field(self):
        self.form.cleaned_data['name'] = 'www.bar.com.'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_data_field(self):
        self.form.cleaned_data['data'] = 'foo'
        self.form.clean()
        self.assert_error('data', self.MSG_INVALID_IPV6)


class CNAMERecordFormTests(base.BaseRecordFormCleanTests):

    CNAME = 'bar.foo.com.'

    def setUp(self):
        super(CNAMERecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'CNAME'
        self.form.cleaned_data['name'] = self.HOSTNAME
        self.form.cleaned_data['data'] = self.CNAME

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_valid_name_field_wild_card(self):
        self.form.cleaned_data['name'] = '*'
        self.form.clean()
        self.assert_no_errors()

    def test_missing_name_field(self):
        self.form.cleaned_data['name'] = ''
        self.form.clean()
        self.assert_required_error('name')

    def test_missing_data_field(self):
        self.form.cleaned_data['data'] = ''
        self.form.clean()
        self.assert_required_error('data')

    def test_invalid_name_field(self):
        self.form.cleaned_data['name'] = '$#%#$foo!!!'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_starting_dash(self):
        self.form.cleaned_data['name'] = '-ww.foo.com'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_trailing_dash(self):
        self.form.cleaned_data['name'] = 'www.foo.co-'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_bad_wild_card(self):
        self.form.cleaned_data['name'] = 'derp.*.' + self.DOMAIN_NAME
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_outside_of_domain_name_field(self):
        self.form.cleaned_data['name'] = 'www.bar.com.'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_data_field(self):
        self.form.cleaned_data['data'] = 'foo'
        self.form.clean()
        self.assert_error('data', self.MSG_INVALID_HOSTNAME_SHORT)


class MXRecordFormTests(base.BaseRecordFormCleanTests):

    MAIL_SERVER = 'mail.foo.com.'
    PRIORITY = 10

    def setUp(self):
        super(MXRecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'MX'
        self.form.cleaned_data['data'] = self.MAIL_SERVER
        self.form.cleaned_data['priority'] = self.PRIORITY

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_missing_data_field(self):
        self.form.cleaned_data['data'] = ''
        self.form.clean()
        self.assert_required_error('data')

    def test_missing_priority_field(self):
        self.form.cleaned_data['priority'] = None
        self.form.clean()
        self.assert_required_error('priority')

    def test_invalid_data_field(self):
        self.form.cleaned_data['data'] = 'foo'
        self.form.clean()
        self.assert_error('data', self.MSG_INVALID_HOSTNAME_SHORT)

    def test_default_assignment_name_field(self):
        self.form.clean()
        self.assertEqual(self.DOMAIN_NAME, self.form.cleaned_data['name'])


class TXTRecordFormTests(base.BaseRecordFormCleanTests):

    TEXT = 'Lorem ipsum'

    def setUp(self):
        super(TXTRecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'TXT'
        self.form.cleaned_data['name'] = self.HOSTNAME
        self.form.cleaned_data['txt'] = self.TEXT

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_valid_name_field_wild_card(self):
        self.form.cleaned_data['name'] = '*'
        self.form.clean()
        self.assert_no_errors()

    def test_missing_name_field(self):
        self.form.cleaned_data['name'] = ''
        self.form.clean()
        self.assert_no_errors()
        self.assertIsNotNone(self.form.cleaned_data['name'])

    def test_missing_txt_field(self):
        self.form.cleaned_data['txt'] = ''
        self.form.clean()
        self.assert_required_error('txt')

    def test_invalid_name_field(self):
        self.form.cleaned_data['name'] = 'foo-'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_starting_dash(self):
        self.form.cleaned_data['name'] = '-ww.foo.com'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_trailing_dash(self):
        self.form.cleaned_data['name'] = 'www.foo.co-'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_invalid_name_field_bad_wild_card(self):
        self.form.cleaned_data['name'] = 'derp.*'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_outside_of_domain_name_field(self):
        self.form.cleaned_data['name'] = 'www.bar.com.'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_HOSTNAME)

    def test_default_assignment_data_field(self):
        self.form.clean()
        self.assertEqual(self.TEXT, self.form.cleaned_data['data'])


class SRVRecordFormTests(base.BaseRecordFormCleanTests):

    SRV_NAME = '_foo._tcp.'
    SRV_DATA = '1 1 srv.foo.com.'
    PRIORITY = 10

    MSG_INVALID_SRV_NAME = 'Enter a valid SRV name'
    MSG_INVALID_SRV_DATA = 'Enter a valid SRV record'

    def setUp(self):
        super(SRVRecordFormTests, self).setUp()
        self.form.cleaned_data['type'] = 'SRV'
        self.form.cleaned_data['name'] = self.SRV_NAME
        self.form.cleaned_data['data'] = self.SRV_DATA
        self.form.cleaned_data['priority'] = self.PRIORITY

    def test_valid_field_values(self):
        self.form.clean()
        self.assert_no_errors()

    def test_missing_name_field(self):
        self.form.cleaned_data['name'] = ''
        self.form.clean()
        self.assert_required_error('name')

    def test_missing_data_field(self):
        self.form.cleaned_data['data'] = ''
        self.form.clean()
        self.assert_required_error('data')

    def test_missing_priority_field(self):
        self.form.cleaned_data['priority'] = None
        self.form.clean()
        self.assert_required_error('priority')

    def test_invalid_name_field(self):
        self.form.cleaned_data['name'] = 'foo'
        self.form.clean()
        self.assert_error('name', self.MSG_INVALID_SRV_NAME)

    def test_invalid_data_field(self):
        self.form.cleaned_data['data'] = 'foo'
        self.form.clean()
        self.assert_error('data', self.MSG_INVALID_SRV_DATA)

    def test_default_assignment_name_field(self):
        self.form.clean()
        self.assertEqual(self.SRV_NAME + self.DOMAIN_NAME,
                         self.form.cleaned_data['name'])



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from django import urls
from django.utils.translation import ugettext_lazy as _  # noqa
from django.utils.translation import ungettext_lazy

from horizon import messages
from horizon import tables
from horizon.utils import memoized

from designatedashboard import api

from openstack_dashboard import policy
from oslo_log import log as logging

LOG = logging.getLogger(__name__)

EDITABLE_RECORD_TYPES = (
    "A",
    "AAAA",
    "CNAME",
    "MX",
    "PTR",
    "SPF",
    "SRV",
    "SSHFP",
    "TXT",
)


class CreateDomain(tables.LinkAction):

    '''Link action for navigating to the CreateDomain view.'''
    name = "create_domain"
    verbose_name = _("Create Domain")
    url = "horizon:project:dns_domains:create_domain"
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("dns", "create_domain"),)

    @memoized.memoized_method
    def allowed(self, request, datum):
        if policy.check((("dns", "get_quota"),), request):
            try:
                if self.table:
                    quota = api.designate.quota_get(request)
                    return quota['domains'] > len(self.table.data)
            except Exception:
                msg = _("The quotas could not be retrieved.")
                messages.warning(request, msg)
        return True


class EditDomain(tables.LinkAction):

    '''Link action for navigating to the UpdateDomain view.'''
    name = "edit_domain"
    verbose_name = _("Edit Domain")
    url = "horizon:project:dns_domains:update_domain"
    classes = ("ajax-modal", "btn-edit")
    policy_rules = (("dns", "update_domain"),)


class ManageRecords(tables.LinkAction):

    '''Link action for navigating to the ManageRecords view.'''
    name = "manage_records"
    verbose_name = _("Manage Records")
    url = "horizon:project:dns_domains:records"
    classes = ("btn-edit")
    policy_rules = (("dns", "get_records"),)


class DeleteDomain(tables.BatchAction):

    '''Batch action for deleting domains.'''
    name = "delete"
    classes = ('btn-danger', 'btn-delete')
    policy_rules = (("dns", "delete_domain"),)

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Domain",
            u"Delete Domains",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Domain",
            u"Deleted Domains",
            count
        )

    def action(self, request, domain_id):
        api.designate.domain_delete(request, domain_id)


class CreateRecord(tables.LinkAction):

    '''Link action for navigating to the CreateRecord view.'''
    name = "create_record"
    verbose_name = _("Create Record")
    classes = ("ajax-modal", "btn-create")
    policy_rules = (("dns", "create_record"),)

    def get_link_url(self, datum=None):
        url = "horizon:project:dns_domains:create_record"
        return urls.reverse(url, kwargs=self.table.kwargs)


class EditRecord(tables.LinkAction):

    '''Link action for navigating to the UpdateRecord view.'''
    name = "edit_record"
    verbose_name = _("Edit Record")
    classes = ("ajax-modal", "btn-edit")
    policy_rules = (("dns", "update_record"),)

    def get_link_url(self, datum=None):
        url = "horizon:project:dns_domains:update_record"
        kwargs = {
            'domain_id': datum.domain_id,
            'record_id': datum.id,
        }

        return urls.reverse(url, kwargs=kwargs)

    def allowed(self, request, record=None):
        return record.type in EDITABLE_RECORD_TYPES


class DeleteRecord(tables.DeleteAction):

    '''Link action for navigating to the UpdateRecord view.'''
    policy_rules = (("dns", "delete_record"),)

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Record",
            u"Delete Records",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Record",
            u"Deleted Records",
            count
        )

    def delete(self, request, record_id):
        domain_id = self.table.kwargs['domain_id']
        return api.designate.record_delete(request, domain_id, record_id)

    def allowed(self, request, record=None):
        return record.type in EDITABLE_RECORD_TYPES


class BatchDeleteRecord(tables.BatchAction):

    '''Batch action for deleting domain records.'''

    name = "delete"
    classes = ('btn-danger', 'btn-delete')
    policy_rules = (("dns", "delete_record"),)

    @staticmethod
    def action_present(count):
        return ungettext_lazy(
            u"Delete Record",
            u"Delete Records",
            count
        )

    @staticmethod
    def action_past(count):
        return ungettext_lazy(
            u"Deleted Record",
            u"Deleted Records",
            count
        )

    def action(self, request, record_id):
        domain_id = self.table.kwargs['domain_id']
        api.designate.record_delete(request, domain_id, record_id)


class DomainsTable(tables.DataTable):

    '''Data table for displaying domain summary information.'''

    name = tables.Column("name",
                         verbose_name=_("Name"),
                         link=("horizon:project:dns_domains:domain_detail"))

    email = tables.Column("email",
                          verbose_name=_("Email"))

    ttl = tables.Column("ttl",
                        verbose_name=_("TTL"))

    serial = tables.Column("serial",
                           verbose_name=_("Serial"))

    class Meta(object):
        name = "domains"
        verbose_name = _("Domains")
        table_actions = (CreateDomain, DeleteDomain,)
        row_actions = (ManageRecords, EditDomain, DeleteDomain,)


def record__details_link(record):
    '''Returns a link to the view for updating DNS records.'''

    return urls.reverse(
        "horizon:project:dns_domains:view_record",
        args=(record.domain_id, record.id))


class RecordsTable(tables.DataTable):

    '''Data table for displaying summary information for a domains records.'''

    name = tables.Column("name",
                         verbose_name=_("Name"),
                         link=record__details_link,
                         )

    type = tables.Column("type",
                         verbose_name=_("Type")
                         )

    data = tables.Column("data",
                         verbose_name=_("Data")
                         )

    priority = tables.Column("priority",
                             verbose_name=_("Priority"),
                             )

    ttl = tables.Column("ttl",
                        verbose_name=_("TTL")
                        )

    class Meta(object):
        name = "records"
        verbose_name = _("Records")
        table_actions = (CreateRecord,)
        row_actions = (EditRecord, DeleteRecord,)
        multi_select = False



# -*- coding: utf-8 -*-

# Copyright 2010-2011 OpenStack Foundation
# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import fixtures
import testtools

from openstack_dashboard.test import helpers as test

from designatedashboard.dashboards.project.dns_domains import forms


_TRUE_VALUES = ('True', 'true', '1', 'yes')


class TestCase(testtools.TestCase):

    """Test case base class for all unit tests."""
    use_mox = False

    def setUp(self):
        """Run before each test method to initialize test environment."""

        super(TestCase, self).setUp()
        test_timeout = os.environ.get('OS_TEST_TIMEOUT', 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            # If timeout value is invalid do not set a timeout.
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        self.useFixture(fixtures.NestedTempfile())
        self.useFixture(fixtures.TempHomeDir())

        if os.environ.get('OS_STDOUT_CAPTURE') in _TRUE_VALUES:
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if os.environ.get('OS_STDERR_CAPTURE') in _TRUE_VALUES:
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))

        self.log_fixture = self.useFixture(fixtures.FakeLogger())


class BaseRecordFormCleanTests(test.TestCase):

    DOMAIN_NAME = 'foo.com.'
    HOSTNAME = 'www'
    use_mox = False

    MSG_FIELD_REQUIRED = 'This field is required'
    MSG_INVALID_HOSTNAME = 'Enter a valid hostname. The '\
                           'hostname should contain letters '\
                           'and numbers, and be no more than '\
                           '63 characters.'
    MSG_INVALID_HOSTNAME_SHORT = 'Enter a valid hostname'

    def setUp(self):
        super(BaseRecordFormCleanTests, self).setUp()

        # Request object with messages support
        self.request = self.factory.get('', {})

        # Set-up form instance
        kwargs = {}
        kwargs['initial'] = {'domain_name': self.DOMAIN_NAME}
        self.form = forms.RecordCreate(self.request, **kwargs)
        self.form._errors = {}
        self.form.cleaned_data = {
            'domain_name': self.DOMAIN_NAME,
            'name': '',
            'data': '',
            'txt': '',
            'priority': None,
            'ttl': None,
        }

    def assert_no_errors(self):
        self.assertEqual(self.form._errors, {})

    def assert_error(self, field, msg):
        self.assertIn(msg, self.form._errors[field])

    def assert_required_error(self, field):
        self.assert_error(field, self.MSG_FIELD_REQUIRED)



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from django.urls import reverse
from django.urls import reverse_lazy
from django.utils.translation import ugettext_lazy as _  # noqa

from horizon import exceptions
from horizon import forms
from horizon import tables
from horizon.views import HorizonTemplateView   # noqa

from openstack_dashboard.api.neutron import tenant_floating_ip_list
from openstack_dashboard.api.nova import server_list

from designatedashboard import api

from .forms import DomainCreate  # noqa
from .forms import DomainUpdate  # noqa
from .forms import RecordCreate  # noqa
from .forms import RecordUpdate  # noqa
from .tables import DomainsTable  # noqa
from .tables import RecordsTable  # noqa
from .utils import limit_records_to_fips  # noqa


class IndexView(tables.DataTableView):
    table_class = DomainsTable
    template_name = 'project/dns_domains/index.html'

    def get_data(self):
        try:
            return api.designate.domain_list(self.request)
        except Exception:
            exceptions.handle(self.request,
                              _('Unable to retrieve domain list.'))
            return []


class CreateDomainView(forms.ModalFormView):
    form_class = DomainCreate
    template_name = 'project/dns_domains/create_domain.html'
    success_url = reverse_lazy('horizon:project:dns_domains:index')

    def get_object_display(self, obj):
        return obj.ip


class DomainDetailView(HorizonTemplateView):
    template_name = 'project/dns_domains/domain_detail.html'

    def get_context_data(self, **kwargs):
        context = super(DomainDetailView, self).get_context_data(**kwargs)
        domain_id = self.kwargs['domain_id']
        try:
            context["domain"] = api.designate.domain_get(self.request,
                                                         domain_id)
            table = DomainsTable(self.request)
            context["actions"] = table.render_row_actions(context["domain"])
        except Exception:
            redirect = reverse('horizon:project:dns_domains:index')
            exceptions.handle(self.request,
                              _('Unable to retrieve domain record.'),
                              redirect=redirect)
        return context


class UpdateDomainView(forms.ModalFormView):
    form_class = DomainUpdate
    template_name = 'project/dns_domains/update_domain.html'
    success_url = reverse_lazy('horizon:project:dns_domains:index')

    def get_object(self):
        domain_id = self.kwargs['domain_id']
        try:
            return api.designate.domain_get(self.request, domain_id)
        except Exception:
            redirect = reverse('horizon:project:dns_domains:index')
            exceptions.handle(self.request,
                              _('Unable to retrieve domain record.'),
                              redirect=redirect)

    def get_initial(self):
        self.domain = self.get_object()
        return self.domain

    def get_context_data(self, **kwargs):
        context = super(UpdateDomainView, self).get_context_data(**kwargs)
        context["domain"] = self.domain
        return context


class RecordsView(tables.DataTableView):
    table_class = RecordsTable
    template_name = 'project/dns_domains/records.html'

    def get_data(self):
        domain_id = self.kwargs['domain_id']
        records = []
        try:
            self.domain = api.designate.domain_get(self.request, domain_id)
            self.servers = api.designate.server_list(self.request, domain_id)
            records = api.designate.record_list(self.request, domain_id)
        except Exception:
            redirect = reverse('horizon:project:dns_domains:index')
            exceptions.handle(self.request,
                              _('Unable to retrieve record list.'),
                              redirect=redirect)

        return records

    def get_context_data(self, **kwargs):
        context = super(RecordsView, self).get_context_data(**kwargs)
        context['domain'] = self.domain
        context['servers'] = self.servers

        return context


class BaseRecordFormView(forms.ModalFormView):
    cancel_label = _("Cancel")

    def get_success_url(self):
        return reverse('horizon:project:dns_domains:records',
                       args=(self.kwargs['domain_id'],))

    def get_domain(self):
        domain_id = self.kwargs['domain_id']
        try:
            return api.designate.domain_get(self.request, domain_id)
        except Exception:
            redirect = reverse('horizon:project:dns_domains:records',
                               args=(self.kwargs['domain_id'],))
            exceptions.handle(self.request,
                              ('Unable to retrieve domain record.'),
                              redirect=redirect)
            # NotAuthorized errors won't be redirected automatically. Need
            # to force the issue
            raise exceptions.Http302(redirect)

    def get_initial(self):
        self.domain = self.get_domain()
        results = {'domain_id': self.domain.id,
                   'domain_name': self.domain.name, }
        if limit_records_to_fips():
            results.update({'fips': tenant_floating_ip_list(self.request),
                            'instances': server_list(self.request)[0]})
        return results

    def get_context_data(self, **kwargs):
        """Set the cancel url

        the cancel_url needs a variable in it
        so we cannot do this with a simple class attr
        this is critical to perform before the super.get_context_data
        """
        self.cancel_url = reverse('horizon:project:dns_domains:records',
                                  args=(self.kwargs['domain_id'],))
        context = super(BaseRecordFormView, self).get_context_data(**kwargs)
        context['domain'] = self.domain
        return context


class CreateRecordView(BaseRecordFormView):
    form_class = RecordCreate
    submit_label = _("Create Record")
    template_name = 'project/dns_domains/create_record.html'


class ViewRecordDetailsView(HorizonTemplateView):
    template_name = 'project/dns_domains/record_detail.html'

    def get_record(self):
        domain_id = self.kwargs['domain_id']
        record_id = self.kwargs['record_id']
        try:
            return api.designate.record_get(self.request, domain_id, record_id)
        except Exception:
            redirect = reverse('horizon:project:dns_domains:records',
                               args=(self.kwargs['domain_id'],))
            exceptions.handle(self.request,
                              _('Unable to retrieve domain record.'),
                              redirect=redirect)

    def get_context_data(self, **kwargs):
        context = super(ViewRecordDetailsView, self).get_context_data(**kwargs)
        self.record = self.get_record()
        context["record"] = self.record
        context["domain_id"] = self.kwargs['domain_id']
        return context


class UpdateRecordView(BaseRecordFormView):
    form_class = RecordUpdate
    submit_label = _("Update Record")
    template_name = 'project/dns_domains/update_record.html'

    def get_record(self):
        domain_id = self.kwargs['domain_id']
        record_id = self.kwargs['record_id']

        try:
            return api.designate.record_get(self.request, domain_id, record_id)
        except Exception:
            redirect = reverse('horizon:project:dns_domains:records',
                               args=(self.kwargs['domain_id'],))
            exceptions.handle(self.request,
                              _('Unable to retrieve domain record.'),
                              redirect=redirect)

    def get_initial(self):
        initial = super(UpdateRecordView, self).get_initial()
        self.record = self.get_record()

        initial.update({
            'id': self.record.id,
            'name': self.record.name.replace("." + initial['domain_name'], ''),
            'data': self.record.data,
            'txt': self.record.data,
            'priority': self.record.priority,
            'ttl': self.record.ttl,
            'type': self.record.type.lower(),
            'description': self.record.description,
        })
        if limit_records_to_fips():
            initial.update({'ip_addr': self.record.data})

        return initial

    def get_context_data(self, **kwargs):
        context = super(UpdateRecordView, self).get_context_data(**kwargs)
        context["record"] = self.record
        return context



# Copyright 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from django.conf.urls import url  # noqa

from .views import CreateDomainView  # noqa
from .views import CreateRecordView  # noqa
from .views import DomainDetailView  # noqa
from .views import IndexView  # noqa
from .views import RecordsView  # noqa
from .views import UpdateDomainView  # noqa
from .views import UpdateRecordView  # noqa
from .views import ViewRecordDetailsView  # noqa


urlpatterns = [
    url(r'^$',
        IndexView.as_view(),
        name='index'),
    url(r'^create/$',
        CreateDomainView.as_view(),
        name='create_domain'),
    url(r'^(?P<domain_id>[^/]+)/update$',
        UpdateDomainView.as_view(),
        name='update_domain'),
    url(r'^(?P<domain_id>[^/]+)$',
        DomainDetailView.as_view(),
        name='domain_detail'),
    url(r'^(?P<domain_id>[^/]+)/records$',
        RecordsView.as_view(),
        name='records'),
    url(r'^(?P<domain_id>[^/]+)/records/create$',
        CreateRecordView.as_view(),
        name='create_record'),
    url(r'^(?P<domain_id>[^/]+)/records/(?P<record_id>[^/]+)/update$',
        UpdateRecordView.as_view(),
        name='update_record'),
    url(r'^(?P<domain_id>[^/]+)/records/(?P<record_id>[^/]+)/$',
        ViewRecordDetailsView.as_view(),
        name='view_record'),
]
